@@ -1,7 +1,7 @@
 /*
     protocol.c -- handle the meta-protocol, basic functions
     Copyright (C) 1999-2005 Ivo Timmermans,
-                  2000-2013 Guus Sliepen <guus@tinc-vpn.org>
+                  2000-2016 Guus Sliepen <guus@tinc-vpn.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -75,10 +75,11 @@ bool send_request(connection_t *c, const char *format, ...) {
 	   input buffer anyway */
 
 	va_start(args, format);
-	len = vsnprintf(buffer, MAXBUFSIZE, format, args);
+	len = vsnprintf(buffer, sizeof buffer, format, args);
+	buffer[sizeof buffer - 1] = 0;
 	va_end(args);
 
-	if(len < 0 || len > MAXBUFSIZE - 1) {
+	if(len < 0 || len > sizeof buffer - 1) {
 		logger(LOG_ERR, "Output buffer overflow while sending request to %s (%s)",
 			   c->name, c->hostname);
 		return false;
@@ -125,20 +126,6 @@ void forward_request(connection_t *from) {
 bool receive_request(connection_t *c) {
 	int request;
 
-	if(c->outgoing && proxytype == PROXY_HTTP && c->allow_request == ID) {
-		if(!c->buffer[0] || c->buffer[0] == '\r')
-			return true;
-		if(!strncasecmp(c->buffer, "HTTP/1.1 ", 9)) {
-			if(!strncmp(c->buffer + 9, "200", 3)) {
-				logger(LOG_DEBUG, "Proxy request granted");
-				return true;
-			} else {
-				logger(LOG_DEBUG, "Proxy request rejected: %s", c->buffer + 9);
-				return false;
-			}
-		}
-	}
-
 	if(sscanf(c->buffer, "%d", &request) == 1) {
 		if((request < 0) || (request >= LAST) || !request_handlers[request]) {
 			ifdebug(META)
