@@ -1,7 +1,7 @@
 /*
     protocol_edge.c -- handle the meta-protocol, edges
     Copyright (C) 1999-2005 Ivo Timmermans,
-                  2000-2012 Guus Sliepen <guus@tinc-vpn.org>
+                  2000-2016 Guus Sliepen <guus@tinc-vpn.org>
                   2009      Michael Tokarev <mjt@corpit.ru>
 
     This program is free software; you can redistribute it and/or modify
@@ -125,8 +125,18 @@ bool add_edge_h(connection_t *c) {
 			} else {
 				ifdebug(PROTOCOL) logger(LOG_WARNING, "Got %s from %s (%s) which does not match existing entry",
 						   "ADD_EDGE", c->name, c->hostname);
-				edge_del(e);
-				graph();
+				e->options = options;
+				if(sockaddrcmp(&e->address, &address)) {
+					sockaddrfree(&e->address);
+					e->address = address;
+				}
+				if(e->weight != weight) {
+					avl_node_t *node = avl_unlink(edge_weight_tree, e);
+					e->weight = weight;
+					avl_insert_node(edge_weight_tree, node);
+				}
+
+				goto done;
 			}
 		} else
 			return true;
@@ -150,6 +160,7 @@ bool add_edge_h(connection_t *c) {
 	e->weight = weight;
 	edge_add(e);
 
+done:
 	/* Tell the rest about the new edge */
 
 	if(!tunnelserver)
